/**
 * Class TList represents a list of generic <T>; it is our own version
 * of ArrayList<T>.
 * You create the list entries by using Add(), one at a time.
 * Once you have a list, you can print it.
 *
 * You can get the nth value with get(n), or set the nth value with set(n, val).
 * If n is out of range, get(n) returns 0 and set(n, val) does nothing.
 *
 */



class TList<T> {
    // instance variables - replace the example below with your own
    private Object[] elements;      // java cannot create an array T[]
    private int currsize;

    /**
     * Constructor for objects of class TList
     */
    public TList()
    {
        // initialise instance variables
        final int defaultCapacity = 5;
        currsize = 0;
        elements = new Object[defaultCapacity];
    }

    /**
     * parameter constructor, allowing you to predefine a capacity.
     */
    public TList(int capacity) {
        currsize = 0;
        elements = new Object[capacity];
    }

    /**
     * Add() Adds the value y to the list, at position currsize
     * (that is, at the end of the existing list).
     * The size of the list is incremented.
     * If pos = nums.length, there is no more room, so this version doesn't do anything;
     * THAT PART YOU ARE TO FIX!!
     */
    public void add(T y) {
        if (currsize == elements.length) {
            expand();
            System.out.println("no more room to add " + y);     // fix this to increase space instead!
            return;     // expand does nothing now
        }
        elements[currsize] = y;
        currsize += 1;      // or currsize++
    }

    private void expand() {
        int newcapacity = elements.length;      // this doesnt do any expanding!
        Object[] newelements = new Object[newcapacity];
        return;
    }

    // get nth value, with range check
    @SuppressWarnings("unchecked")
    public T get(int n) {
        if (n<0 || n >= currsize) {
            System.out.println("Warning: TList.get() called with out-of-range n = " + n);
            return null;
        }
        return (T) elements[n];         // NOTE THE (T) CAST from Object to T!
    }

    // set nth value, with range check
    public void set(int n, T val) {
        if (n<0 || n >= currsize) {
            System.out.println("Warning: Strlist.set() called with out-of-range n = " + n);
            return;
        }
        elements[n] = val;
    }

    public int size() {
        return currsize;
    }


    public void print() {
        int i=0;
        while (i<currsize) {
            System.out.println(i + ":  " + elements[i]);
            i++;
        }
    }

    public boolean equals(Object obj) {
        if (obj == null) return false;

        // If parameter cannot be cast to TList return false.
        if (! (obj instanceof TList)) return false;
        TList s2 = (TList) obj; // downcast

        // now we check the actual data:
        if (currsize != s2.currsize) return false;
        for (int i=0; i< currsize; i++) {
            if (!(elements[i].equals(s2.elements[i]))) return false;
            //if (!(elements[i]==s2.elements[i])) return false;
        }
        return true;
    }

    public int hashCode() {
        int hc = 0;
        for (int i = 0; i< currsize; i++) {
            hc = hc ^ elements[i].hashCode();
        }
        return hc;
    }
}